
# q-expansion of newform 7744.2.a.n, downloaded from the LMFDB on 02 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 7744
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-1], [3], [0], [0], [0], [0], [0], [9], [0], [5], [-7], [0], [0], [12], [-6], [-15], [0], [13], [3], [0], [0], [0], [-9], [17], [0], [4], [0], [0], [21], [0], [0], [-3], [0], [0], [0], [23], [-16], [0], [0], [21], [25], [15], [0], [0], [20], [0], [1], [0], [5], [0], [0], [0], [-27], [18], [0], [-30], [0], [0], [0], [0], [0], [0], [12], [19], [27], [35], [0], [0], [0], [-9], [0], [37], [0], [-25], [-39], [15], [-2], [30], [0], [24], [-10], [0], [29], [0], [21], [39], [0], [0], [31], [-3], [0], [-43], [0], [40], [0], [45], [-15], [0], [0], [0], [0], [0], [0], [0], [-47], [-48], [0], [36], [0], [0], [0], [-42], [-1], [-7], [-45], [41], [27], [-51], [0], [-13], [0], [0], [24], [17], [0], [-19], [-51], [53], [0], [0], [0], [23], [38], [0], [0], [-54], [0], [3], [0], [0], [0], [49], [0], [-29], [45], [0], [0], [-31], [36], [0], [57], [-56], [0], [8], [60], [0], [30], [0], [0], [57], [0], [0], [-45], [-27], [-51], [20], [0], [0], [9], [0], [-50], [0], [0], [-41], [-15], [0], [-63], [0], [0], [-28], [0], [59], [0], [-21], [-6], [0], [-64], [0], [0], [61], [0], [-47], [-18], [0], [-69], [0], [1], [0], [0], [0], [0], [58], [0], [-69], [-57], [-65], [0], [0], [40], [0], [45], [-19], [72], [0], [-53], [0], [0], [12], [-39], [0], [0], [-75], [-29], [0], [0], [54], [60], [0], [0], [31], [0], [0], [0], [-71], [0], [-59], [0], [-45], [15], [-69], [0], [-76], [-61], [0], [0], [-52], [72], [0], [0], [0], [0], [-57], [10], [0], [0], [70], [0], [27], [0], [0], [0], [0], [0], [-78], [-80], [81], [-3], [0], [0], [67], [83], [-26], [0], [0], [-79], [63], [0], [0], [0], [0], [-85], [0], [0], [0], [75], [-34], [0], [71], [0], [27], [-48], [0], [0], [0], [0], [-73], [81], [0], [0], [0], [0], [0], [24], [-5], [7], [87], [-85], [-84], [0], [0], [-90], [0], [-59], [0], [0], [39], [0], [46], [0], [0], [-23], [0], [-61], [0], [-25], [79], [-89], [0], [0], [0], [-18], [0], [0], [0], [57], [-65], [0], [95], [0], [91], [-93], [0], [-35], [9], [0], [-83], [-37], [0], [-87], [0], [-97], [-30], [0], [0], [0], [75], [0], [0], [-21], [0], [0], [-63], [0], [43], [-71], [0], [0], [0], [57], [-16], [0], [0], [0], [47], [96], [0], [0], [0], [0], [78], [-81], [0], [0], [0], [9], [-28], [103], [0], [-87], [-91], [-39], [-15], [-53], [-63], [0], [0], [90], [32], [0], [0], [21], [0], [-102], [0], [0], [-98], [0], [-104], [-45], [0], [0], [96], [0], [0], [75], [0], [-107], [18], [0], [69], [0], [108], [0], [74], [93], [51], [0], [-97], [0]]
